<?php

namespace Drupal\private_message_flood\Plugin\PrivateMessageConfigForm;

use Drupal\Core\Form\FormStateInterface;
use Drupal\private_message\Plugin\PrivateMessageConfigForm\PrivateMessageConfigFormBase;
use Drupal\private_message\Plugin\PrivateMessageConfigForm\PrivateMessageConfigFormPluginInterface;

/**
 * Adds Private Message Flood settings to the Private Message config page.
 *
 * @PrivateMessageConfigForm(
 *   id = "private_message_flood_settings",
 *   name = @Translation("Private Message Flood Protection settings"),
 * )
 */
class PrivateMessageFloodPrivateMessageConfigForm extends PrivateMessageConfigFormBase implements PrivateMessageConfigFormPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function buildForm(FormStateInterface $formState) {

    $form['header'] = [
      '#prefix' => '<p>',
      '#suffix' => '</p>',
      '#markup' => $this->t('Flood protection limits the number of post users can make in a given period of time. You can apply flood protection on a per-role basis below. Priority is given to roles from top to bottom. The highest priority role a user is found to have, will be checked against the flood protection settings for that role when determining if they have flooded the private message system.'),
    ];

    $group_class = 'group-order-weight';

    $roles = array_map(['\Drupal\Component\Utility\Html', 'escape'], user_role_names(TRUE));
    $items = [];
    foreach ($roles as $role_id => $role_name) {
      $config = $this->configFactory->get('private_message_flood.role.' . $role_id);

      $items[$role_id] = [
        'label' => $role_name,
        'weight' => $config->get('weight'),
        'elements' => [
          'limit' => [
            '#type' => 'number',
            '#title' => t('Maxium number of messages'),
            '#description' => $this->t('Leave empty to disable flood protection for this role'),
            '#default_value' => $config->get('limit'),
          ],
          'duration' => [
            '#type' => 'duration',
            '#title' => t('Duration'),
            '#granularity' => 'y:d:m:h:i:s',
            '#default_value' => $config->get('duration'),
          ],
        ],
      ];
    }

    $form['roles'] = [
      '#type' => 'table',
      '#caption' => $this->t('Roles'),
      '#header' => [
        $this->t('Label'),
        $this->t('Values'),
        $this->t('Weight'),
      ],
      '#tableselect' => FALSE,
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => $group_class,
        ],
      ],
    ];

    // Build rows.
    foreach ($items as $key => $value) {
      $form['roles'][$key]['#attributes']['class'][] = 'draggable';
      $form['roles'][$key]['#weight'] = $value['weight'];

      // Label col.
      $form['roles'][$key]['label'] = [
        '#plain_text' => $value['label'],
      ];

      // Values col.
      $form['roles'][$key]['values'] = [
        'details' => [
          '#type' => 'details',
          '#title' => $this->t('Click to expand'),
          '#open' => FALSE,
          'elements' => $value['elements'],
        ],
      ];

      // Weight col.
      $form['roles'][$key]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', ['@title' => $value['label']]),
        '#title_display' => 'invisible',
        '#default_value' => $value['weight'],
        '#attributes' => ['class' => [$group_class]],
      ];
    }

    uasort($form['roles'], ['Drupal\Component\Utility\SortArray', 'sortByWeightProperty']);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array $values) {

    foreach ($values['roles'] as $rid => $data) {
      $this->configFactory->getEditable('private_message_flood.role.' . $rid)
        ->set('limit', $data['values']['details']['elements']['limit'])
        ->set('duration', $data['values']['details']['elements']['duration'])
        ->set('weight', $data['weight'])
        ->save();
    }
  }

}
